<?php
/**
 * Vytváří virtuální přístupové metody
 */

abstract class Accessor
{
	

	/** Metoda zjistuje, jestli je volana funkce vrat* nebo nastav*
	 * podle toho vola prislusnou virtualni nastav nebo vrat funkci
	 * tato funkce je volána pouze, pokud není přetížena v potomkovi	
	 * @param string $funkce
	 * @param mixed $argumenty
	 */
	
		
	public function __call($funkce, $argumenty)
	{
		$typPristupoveMetody = substr($funkce, 0, 4);
		if ($typPristupoveMetody == "vrat")
		{
			$atribut = substr($funkce, 4);
			return($this->pristupovaMetodaGet($atribut,$argumenty));
		}
	
		$typPristupoveMetody = substr($funkce, 0, 6);
		if ($typPristupoveMetody == "nastav")
		{
			$atribut = substr($funkce, 6);
			return($this->pristupovaMetodaSet($atribut, $argumenty));
		}
	
		throw new Chyba("Vnitřní chyba - metoda [$funkce] neexistuje nebo není přístupná.");
	}
	
	/** Metoda zjistuje, jestli je volana funkce vrat* nebo nastav* pro obsluhu staticky atributu
	* podle toho vola prislusnou virtualni nastav nebo vrat funkci
	* tato funkce je volána pouze, pokud není přetížena v potomkovi
	* @param string $funkce
	* @param mixed $argumenty
	*/
	
	public static function __callStatic($funkce, $argumenty)
	{
		 
		$typPristupoveMetody = substr($funkce, 0, 4);
		if ($typPristupoveMetody == "vrat")
		{
			$atribut = substr($funkce, 4);
			return(self::pristupovaMetodaGetStatic($atribut));
		}
	
		$typPristupoveMetody = substr($funkce, 0, 6);
		if ($typPristupoveMetody == "nastav")
		{
			$atribut = substr($funkce, 6);
			return(self::pristupovaMetodaSetStatic($atribut, $argumenty));
		}
		throw new Chyba("Vnitřní chyba - metoda [$funkce] neexistuje nebo není přístupná.");
	}
	 
	//////////////////////////////////////////////////////////////////////////////////////////////
	/* Metoda vraci staticky atribut podrizeneho objektu
	 *
	* $atribut string
	*/
	
 private static function pristupovaMetodaGetStatic($atribut)
   {
   	$atribut = lcfirst($atribut);
   	$trida =  get_called_class(); 
   
   	if (property_exists($trida, $atribut))
   	{
   		return  static::$$atribut;
   	}
   	else
   	{
   		throw new Chyba("Vnitřní chyba - atribut [$atribut] neexistuje.");
   	}
   }
   
   //////////////////////////////////////////////////////////////////////////////////////////////
   /* Virtualni pristupova metoda nastavuje staticky atribut objektu na pozadavanou hodnotu
    * atribut str
   *  hodnota void
   */
    
   private static function pristupovaMetodaSetStatic($atribut, $hodnota)
   {  
   	$hodnota = $hodnota[0];
   	$atribut = lcfirst($atribut);
   	$trida =  get_called_class();
   	
   	if (property_exists($trida, $atribut))
   	{
   		static::$$atribut =  $hodnota;
   	}
   	else
   	{
   		throw new Chyba("Vnitřní chyba - atribut [$atribut] neexistuje.");
   	}
   }
   
   
   

   //////////////////////////////////////////////////////////////////////////////////////////////  
   /* Virtualni pristupova metoda nastavuje atribut objektu na pozadavanou hodnotu
    * atribut str
    * hodnota void
    */
   
   private function pristupovaMetodaSet($atribut, $hodnota) 
   {
    
   	$hodnota = isset ($hodnota[0]) ? $hodnota[0] : null;
  	$atribut = lcfirst($atribut);
   	if (property_exists($this, $atribut)) 
   	{       	
        $this->$atribut =  $hodnota;        
   	} 
   	else 
   	{
   		throw new Chyba("Vnitřní chyba - atribut [$atribut] neexistuje.");
   	}
   }
   
   //////////////////////////////////////////////////////////////////////////////////////////////
   /* Virtualni pristupova metoda vraci hodnotu atributu
    * atribut str
   *  hodnota void
   */
   
   private function pristupovaMetodaGet($atribut,$argumenty=null) 
   {  
   	$atribut = lcfirst($atribut);    
    if (property_exists($this, $atribut)) 
   	{	
   		return $this->$atribut;
   	} 
    else 
   	{   		
   	  throw new Chyba("Vnitřní chyba - atribut [$atribut] neexistuje.");
   	}
   } 


}
