<?php
class DB {

	// Databázové spojení
	private static $spojeni;

	// Výchozí nastavení ovladače
	private static $nastaveni = Array(
			PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
			PDO::MYSQL_ATTR_INIT_COMMAND => "SET NAMES utf8"
	);

	// Připojí se k databázi pomocí daných údajů
	public static function pripoj($host, $uzivatel, $heslo, $databaze, $port=null)
	{
		if (!isset(self::$spojeni))
		{
			try
			{
				self::$spojeni = @new PDO("mysql:host=$host;port=$port;dbname=$databaze",$uzivatel,$heslo,self::$nastaveni);
			}
			catch (PDOException $e)
			{				
				die("Nastala chyba při přístupu do databáze.");
			}
		}
	}


	/**
	 * Spustí dotaz a vrátí z něj první řádek jako asociativní pole
	 * @param string $dotaz
	 * @param array $parametry
	 */
	
	public static function dotazJeden($dotaz, $parametry = array())
	{
		$navrat = self::dotaz($dotaz,$parametry) ;
		return $navrat->fetch(PDO::FETCH_ASSOC);
	}

	/**
	 * Spustí dotaz a vrátí všechny jeho řádky jako pole asociativních polí
	 * @param string $dotaz
	 * @param array $parametry
	 */
	
	public static function dotazVsechny($dotaz, $parametry =  array()) {
		$navrat = self::dotaz($dotaz,$parametry) ;
		return $navrat->fetchAll(PDO::FETCH_ASSOC);
	}

	/**
	 * Spustí dotaz a vrátí pole obsahující hodnoty z prvního sloupce vybraných řádků
	 * @param string $dotaz
	 * @param array $parametry
	 */
	
	public static function dotazJedenSloupec($dotaz, $parametry =  array()) {
		$navrat = self::dotaz($dotaz,$parametry) ;
		return $navrat->fetchAll(PDO::FETCH_COLUMN, 0);
	}

	/**
	 * Spustí dotaz a vrátí  počet ovlivněných řádků
	 * @param string $dotaz
	 * @param array $parametry
	 */
	
	public static function dotazPocet($dotaz,$parametry =  array()) {
		$navrat = self::dotaz($dotaz,$parametry) ;
		return $navrat->rowCount();
	}

	
	/**
	 * Obecné spuštění dotaz
	 * pri __DB_DEBUG__ == true vypisuje SQL dotaz a PDO chybovou hlasku
	 * @param string $dotaz
	 * @param array $parametry
	 * @throws Chyba
	 * @return PDO statement
	 */
	
	public static function dotaz($dotaz, $parametry= array())
	{
	 try
	 {
	 	$casovac = microtime(true);
	  	//echo $dotaz;echo "<br><br>";
	  	//var_dump($parametry);
	 	if (is_array($parametry))
	 	{
	 		$navrat  = self::$spojeni->prepare($dotaz);
	 		$navrat->execute($parametry);
	 	}
	 	else
	 	{
	 		throw new Chyba('Nastala chyba při přístupu do databáze - Chybně zapsaná funkce');
	 	}
	 	$casovac = microtime(true)-$casovac;
	 	//echo $casovac ."<br>";

	 }
	 catch (PDOException $e)
	 {
	 	if (__DB_DEBUG__ and $parametry == null)
	 	{
	 		$chyba = "<br>".htmlspecialchars($dotaz) ." <br> ". $e->getMessage();
	 	}
	 	elseif  (__DB_DEBUG__)
	 	{
	 		$chyba = "<br>".htmlspecialchars(self::dotazPredPrepare($dotaz,$parametry)) ." <br> ". $e->getMessage();
	 	}
	 	else
	 	{
	 		$chyba = null;
	 	}

	 	throw new Chyba('Nastala chyba při přístupu do databáze.'. $chyba );
	 }

		return $navrat;
	}


	/**
	 * Dotaz převádí z formátu pro PDO (dotaz, parametry) do SQL
	 * Slouží jen k snadnějšímu dubugování
	 * Nemusí fungovat pro složitější dotazy
	 * @param string $query
	 * @param array $params
	 * @return string
	 */
	
	private static function dotazPredPrepare($query, $params=array()) {
		$keys = array();
		$values = $params;

		foreach ($params as $key => $value) {
			if (is_string($key)) {
				$keys[] = '/:'.$key.'/';
			} else {
				$keys[] = '/[?]/';
			}

			if (is_string($value))
				$values[$key] = "'" . $value . "'";

			if (is_array($value))
				$values[$key] = implode(',', $value);

			if (is_null($value))
				$values[$key] = 'NULL';
		}

		$query = preg_replace($keys, $values, $query, 5, $count);
		return $query;
	}

	
	/**
	 * Vrátí poslední ID posledního vloženého záznamu
	 */
	public static function posledniId()	{
		return self::$spojeni->lastInsertId();
	}
	
    /**
     * Spuštení transakce
     * Volá se před zadáním dotazů
     */
	
	public static function spustitTransakci() 
	{
		try 
		{
			self::$spojeni->beginTransaction();
		}
		catch (PDOException $e)
		{
			throw new Chyba($e->getMessage());
		}
	}
	
	/**
     * Potvrzení transakce      
     */	
	public static function commit() {
		try 
		{
			
			self::$spojeni->commit();
		}
		catch (PDOException $e)
		{
			throw new Chyba($e->getMessage());
		} 
	}
	
	/**
	 * Zrušit transakci a vzít změny zpět
	 */	
	public static function rollBack() {
	try 
		{
			self::$spojeni->rollBack(); 
		}
		catch (PDOException $e)
		{
			throw new Chyba($e->getMessage());
		}
	}

}