<?php
class Kolekce implements IteratorAggregate{
	
	
 function __construct($povoleneObjektyTridy=null)
 { 	
 	if ($povoleneObjektyTridy)
 	{
		if (!class_exists($povoleneObjektyTridy)) 				
		{
			throw new Chyba ("Do kolekce nelze přiřadit objekty neexistující třídy [$povoleneObjektyTridy]");
 	 	}
 	 	$this->povoleneObjektyTridy = $povoleneObjektyTridy;	 	 	
 	} 	
 }
  

  private $povoleneObjektyTridy = null;			//bude prijim objekty jen teto tridy @string
  public $polozky = array();
                                  //atribut pro uložení názvu callback funkce
  private $nacteno = false;     //příznak indikující jestli již došlo k volání
  private $callback;                              //callback funkce
                              
  private $klicCallback = null;
  
  public function getIterator() 
    {
		  return new ArrayIterator($this->vratPolozky());
    }
  
  
  
  /**
   *  prida objekt do kolekce
   * pokud je $klic nevyplnen stane se klicem id objektu
   * pokud je vyplněn stane se klicem id tento klic
   * pokud je FALSE doplni se autoincrementem   
   */
  
  
  public function pridej($obj, $klic = null) 
  {
  
  	if (isset($this->povoleneObjektyTridy) and get_class($obj)!=$this->povoleneObjektyTridy)
    {
    	throw new Vyjimka("Nepovolené vkládání objektu[".get_class($obj)."] místo[".$this->povoleneObjektyTridy."].");
    }
    	
  	$this->zkontrolujCallback(); 
  	if ($klic === false )//autoincrement
  	{
  		$this->polozky[] = $obj;
  	}    
    else
    {     	
    	if ($this->klicCallback)
    	{
    		
    		$klic =  call_user_func(array($obj,$this->klicCallback));
    	}
    	else 
    	{
    		$klic = ($klic == null) ?  $obj->vratId() : $klic;   //id obj
    		
    	}
    	 if(isset($this->polozky[$klic]))
    	 {
    	 	throw new Chyba("Klíč se už používá -  klíč[$klic] objetkt[".get_class($obj)."].");
    	 }
    	 $this->polozky[$klic] = $obj;
    }   
  }
     
  public function vratPrvni()
  {
  	$this->zkontrolujCallback();
  	if ($this->polozky != array())
  	{	
  		$klice = array_keys($this->polozky);
  		return $this->polozky[$klice[0]];
  	}
  	return null;
  }
      
  	
  public function vratPosledni()
  {  
  	$this->zkontrolujCallback();
  	if ($this->polozky != array())
  	{
  		return end($this->polozky);
  	}
  	return null;
  }
  
  

  public function odstran($klic) {

    $this->zkontrolujCallback();
  	
    if(isset($this->polozky[$klic]))
     {
       unset($this->polozky[$klic]);
     }
     else 
     {     	
     	throw new Chyba("Neplatný klíč \"$klic.\"");
     } 
  }
  
  public function vrat($klic) {

    $this->zkontrolujCallback();

    if(isset($this->polozky[$klic])) 
    {
      return $this->polozky[$klic];
    } 
    else 
    {     
     throw new Chyba("Neplatný klíč \"$klic.\"");
    }
  }

 
  public function vratKlice() {
       $this->zkontrolujCallback();
    return array_keys($this->polozky);
  }  
 
 
  public function vratPocet() {
  	$this->zkontrolujCallback();
    return sizeof($this->polozky);
  }
  
  public function existuje($klic) {  
    $this->zkontrolujCallback();
    return (isset($this->polozky[$klic]));
  }
  
   
  public function vratPolozky()
  {
  	$this->zkontrolujCallback();
  	return $this->polozky;
  }
  
  public function vratPovoleneObjektyTridy()
  {
  	return $this->povoleneObjektyTridy;
  }
  
 
  /**
  * Tuto metodu použijte pro určení funkce, která se zavolá před
  * přístupem ke kolekci. Tato funkce by měla brát kolekci jako 
  * svůj jediný parametr.
  */  
  
  public function nastavKlicCallback ($funkce)
  {	 	
  	if (!$this->povoleneObjektyTridy)
  	{
  		  throw new Chyba("Není možné nastavit callback pro klíč pro kolekci bez omezení objektů"); 
  	}  	 
  	 $this->klicCallback = $funkce;   	
  }
    
  public function nastavCallback($funkce, $volajiciObjekt) 
  	{   
      $callback = array($volajiciObjekt, $funkce);
    
    //ověření správnosti parametrů
    if (!is_callable($callback, false, $nazevVolani)) 
    {
      throw new Chyba("Není možné provést volání  $nazevVolani");     
    }

    $this->callback = $callback;
    
  }
  
  
  /**
   * Zjišťuje, jestli jdou po sobě stejně hodnoty v poli pořadí stejně jako klíče v položkách kolekce
   * Pokud ne, přeskládá položky podle pole pořadí
   * @param array $poradi - array( ? => id1, ? =>id5  )
   * $this->vratKlice() - array( 0 => id1, 1 => id5) 
   */
  public function seradit(array $poradi)
  { 
  	
  
  	if ($poradi != $this->vratKlice())
  	{  	
  		
  		$temp = array();
  		foreach  ($poradi as $id)
  		{
  			//muze se stat ze pozadovany id neni v DB tak se ani nenacte do kolekce
  			if ($this->existuje($id)) {
  				$temp[$id] = $this->vrat($id);
  			}
  		}
  		$this->polozky =  $temp;
  		unset ($temp);
  	}  	 	
  }


  /**
  * Kontrola jestli je nastavena callback funkce. Pokud ano a dosud 
  * se nevolala, zavolá se nyní. 
  */
  private function zkontrolujCallback() {

   if(isset($this->callback) && !$this->nacteno)
     {      
        $this->nacteno = true;
        call_user_func($this->callback, $this);
    }    
  }
}