<?php
class Prihlaseni
{
	private static $uzivatel = null;  // instance prihlaseneho uzivatele
	private static $csrfToken = null;
	
		
	public static  function kontrolaRelacePrihlaseni($modul)
	{		 
		try
		{			
			$modulyBezLoginu = unserialize(__MODULY_BEZ_LOGINU__);
			//načtení tokenu ze session
			self::$csrfToken = isset($_SESSION['uzivatel']["csrfToken"])? $_SESSION['uzivatel']["csrfToken"] : null;			
			
			/**
			 * Tento blok je aktivní jen při povolené csrf ochraně na modulech vyžadujících přihlášení 
			 * Pokud je odeslán post požadavek, kontroluje se platnost tokenu.
			 * Když je token neplatný dojde k přesměrování na stejnou stránku, tím se post požadavek zruší, zároveň je zobrazena chybová zpráva
			 */
			if (__CSRF_OCHRANA__ and $modul !== 'login' and !in_array($modul,$modulyBezLoginu))
			{				
				if ($_POST and (!isset($_POST["csrfToken"]) or ($_POST["csrfToken"] != self::$csrfToken)))
				{
					new Chyba("Naplatný token, akce nebyla dokončena.");
					FK::presmerovat();
				} 
			}
					
			/**
			 * Pokud je session platná a nevypršela doba přihlášení, je aktualizována session
			 * Pokud max. doba přihlášení vypršela, jsou všechny session odstraněny a uživatel je přesměrován na login
			 */			
			
			if (isset($_SESSION['uzivatel']))
			{
				if (($_SESSION['uzivatel']['posledniAktivita'] + __DOBA_AUTOMATICKEHO_ODHLASENI__) < time())
				{
					new Chyba('Byl jste automaticky odhlášen pro dlouhou neaktivitu');
					self::odstranitSession();
				}
				else 
				{
					self::$uzivatel = N::Uzivatel($_SESSION['uzivatel']['id']);
					self::prihlasit();  //aktualizovat
				}				
			}
						
			//kontrola přihlášení, pokud je u modulu vyžadována
		
			if ($modul !== 'login' and !in_array($modul,$modulyBezLoginu) and !self::$uzivatel)
			{	
				FK::presmerovat(__WEB__."login");
			}	
			
						
			
			
			
		
			
		}
		catch(Chyba $e)
		{
			self::odhlasit();
		}
		 
	}
	
	/**
	 * Porovná předaný email a heslo s uádaji v databázi
	 * při úspěchu přihlásit
	 * @param ustring $email
	 * @param string $heslo
	 * @throws Chyba
	 */
	public static function kontrolaPrihlasovacichUdaju($email,$heslo)
	{
		Validace::delka($heslo,6,20,'Heslo');
		Validace::email($email);
		$parametry = array("email"=>$email);
		$dotaz = "SELECT id_uzivatel,heslo FROM uzivatel WHERE email = :email LIMIT 1";
		$radek = DB::dotazJeden($dotaz,$parametry);
		if (!$radek or !password_verify($heslo, $radek["heslo"]))
		{
			throw new Chyba('Neplatný e-mail nebo heslo.');
		}
		self::$uzivatel = N::Uzivatel($radek['id_uzivatel']);
		self::$csrfToken = rand(1, 1e9);
		self::prihlasit();
	}
	
		 
	private static function  prihlasit()
	{		 
		if (self::$uzivatel)
		{		
			session_regenerate_id(); //vygenerovat nové SID - ochrana proto Session Fixation
			$_SESSION['uzivatel'] =  self::$uzivatel->zobraz();
			$_SESSION['uzivatel']['posledniAktivita'] = time();
			$_SESSION['uzivatel']['csrfToken'] = self::$csrfToken;
		}		 
	}
	
	/**
	 * Ruší session a coookies a přesměrovává na login
	 * při volání odhlásit se nezobrazí předchozí chybová hlášení uložené v session
	 */
	
	private static  function odstranitSession()
	{
		$_SESSION = array();
		$params = session_get_cookie_params();
    	setcookie(session_name(), '', time() - 42000,
        $params["path"], $params["domain"],
        $params["secure"], $params["httponly"]);
   	 	session_destroy();
  
	}
	
	public static function  odhlasit()
	{
		self::odstranitSession();
		FK::presmerovat(__WEB__."login") ;
	}
	
	public static function vratUzivatel()
	{
		return self::$uzivatel;
	}
	public static  function vratCsrfToken()
	{
		return self::$csrfToken;
	}
}
