<?php

class Rezervace Extends BO2
 {
  
  protected $id = null;
  protected $zakaznikId = null;
  protected $kontaktId = null;
  protected $datumOd = null; 
  protected $datumDo = null; 
  protected $ubytovaciPoplatekCena = 0; 
  protected $rekreacniPoplatekCena = 0; 
  protected $ubytovaciPoplatekPocet = 0;
  protected $rekreacniPoplatekPocet = 0;
  protected $cenaCelkem = null;
  protected $poznamka = null;
  protected $cisloFaktury = null;
  protected $dobropisCena = null;
  
  protected $kontakt = null;
  protected $strava = null;
  protected $pokojRezervace = null;
  protected $stavy = null;
  
  protected static $kolekce;
  
  protected static $tabulkaDB = "rezervace";
  protected static $mapovaniDB = array ( 
   "id" => "id_rezervace",
   "zakaznikId" => "id_zakaznik",
   "kontaktId" => "id_kontakt",
   "datumOd" => "datum_od",
   "datumDo"  => "datum_do",   
   "ubytovaciPoplatekCena" => "ubyt_popl_cena",
   "rekreacniPoplatekCena"  => "rekr_popl_cena",  
   "ubytovaciPoplatekPocet" => "ubyt_popl_pocet",
   "rekreacniPoplatekPocet" => "rekr_popl_pocet",
   "cenaCelkem" => "cena_celkem",  
   "poznamka" => "poznamka",	
   "cisloFaktury" => "cislo_faktury",
   "dobropisCena" => "cena_dobropis",		
   );
  
  /**pomocné*/
  protected $posledniStavZaplaceni = null;
  
   
	function __construct($id=null)
	{  			
	   parent::__construct($id);		  
	 
      $this->pokojRezervace = new Kolekce('PokojRezervace');
   	  $this->pokojRezervace->nastavCallback('kolekceSouvisejicich', $this);
   	  
   	  $this->strava = new Kolekce('Strava');
   	  $this->strava->nastavCallback('kolekceSouvisejicich', $this);
   	  
   	  $this->sluzby = new Kolekce('Sluzba');
   	  $this->sluzby->nastavCallback('kolekceSouvisejicich', $this);
   	  
   	  $this->stavy = new Kolekce('StavRezervace');
   	  $this->stavy->nastavCallback('kolekceStavuRezervace', $this);  
     
   	  if (!$this->id)
   	  {
   	  	//nastavení výše poplatků při vytváření nové rezervace
   	  	$this->ubytovaciPoplatekCena = N::Nastaveni("ubytovaciPoplatekCena")->vratHodnota();
   	  	$this->rekreacniPoplatekCena = N::Nastaveni("rekreacniPoplatekCena")->vratHodnota();   	  
   	  }
	}	
  

	
	/**
	 * vytvořit kolekci stavů rezervace v pořadí od nejnovějšího
	 * @param Kolekce $kolekce
	 */
	public function kolekceStavuRezervace(Kolekce $kolekce)
	{		
		$dotaz = "SELECT id_stav_rezervace FROM stav_rezervace WHERE id_rezervace = :id ORDER BY cas DESC";
		$parametry = array("id" => $this->id);
		$poleID = DB::dotazJedenSloupec($dotaz,$parametry);
		StavRezervace::kolekce($kolekce, $poleID);
		
		
	}

 
  public function nastavDatum($od,$do,$format)
  {    
     Validace::datum($od,$format);
     Validace::datum($do,$format);
     $od = DateTime::createFromFormat($format, $od);
     $do = DateTime::createFromFormat($format, $do);          
     $od = $od->format("Y-m-d");
     $do = $do->format("Y-m-d");
     
     //porovnávány jsou řetězce, protože porovnání dateTime v některých verzích php zlobí
     if ($od >= $do)
     {
         Throw new Chyba("Datum začátku pobytu musí být menší než datum konce pobytu.");
     }
    
     $this->datumOd = $od;
     $this->datumDo = $do; 
  }
 
    
  public function nastavPoznamka($prom)
  {
  	Validace::delka($prom, 0, 800, "Poznámka");
  	$this->poznamka = $prom;  
  }
  
  public function nastavCisloFaktury($prom)
  {
  	Validace::delka($prom, 1, 50, "Číslo faktury");
  	$parametry["cisloFaktury"] = $prom;
  	$parametry["id"] = $this->id;
  	$dotaz = "SELECT ".self::$mapovaniDB['id']." FROM ".self::$tabulkaDB." WHERE ".self::$mapovaniDB['cisloFaktury']." = :cisloFaktury and ".self::$mapovaniDB['id']."<>:id LIMIT 1";
  	$radek = DB::dotazJeden($dotaz,$parametry);
  	if($radek)
  	{  		
  		throw new Chyba("Stejné číslo faktury již existuje u rezervace č. ".$radek[self::$mapovaniDB['id']].".");
	}
  	
  	$this->cisloFaktury = $prom;
  }
  
   /**cena dobropisu, zaokrouhlená na celá čísla*/
   public function nastavDobropisCena($prom)
   {
   	Validace::numeric($prom,"Cena dobropisu",$nastaveni = array("min" => 0, "max" => $this->vratCenaCelkem()));   	
   	$this->dobropisCena = round($prom);
   }
   
  
   public function nastavRekreacniPoplatekOsvobozenoPocet($prom)
   {   	
   	Validace::integer($prom, "Počet osob, které neplatí rekreační poplatek", array("min" => 0));
   	if ($prom > $this->vratPocetHostu()){
   		$this->rekreacniPoplatekPocet = 0;
   		Throw new Chyba("Počet osob, které platí rekreační poplatek nesmí být větší než celkový počet ubytovaných osob.");   		
   	}    	
   	$this->rekreacniPoplatekPocet = $this->vratPocetHostu() - $prom;   	
   }
   
   public function nastavUbytovaciPoplatekOsvobozenoPocet($prom)
   {
   	Validace::integer($prom, "Počet osob, které neplatí ubytovací poplatek,", array("min" => 0));
   	if ($prom > $this->vratPocetHostu()){
   		$this->ubytovaciPoplatekPocet = 0;
   		Throw new Chyba("Počet osob, které neplatí ubytovací poplatek, nesmí být větší než celkový počet ubytovaných osob.");
   	}
   	$this->ubytovaciPoplatekPocet = $this->vratPocetHostu() - $prom;   
   }
   
   public function nastavCenaCelkem($prom)
   {
   	if (!is_numeric($prom) or $prom < 0) throw new Chyba("Konečná cena musí být kladné číslo");  
   	$this->cenaCelkem = $prom;
   } 
   
   public function nastavSlevaProcenta($prom)
   {
   	$this->nastavCenaCelkem($this->vratCenaCelkemVypoctena() - $this->vratCenaCelkemVypoctena()/100*$prom);
   }
  
  public function vratCenaUbytovaniBezPoplatku()
  {
    $cena = 0;
    foreach  ($this->pokojRezervace as $pokojRezervace)
    {       
       $cena = $cena +  $pokojRezervace->vratCena();        
    }   
    return $cena;
  }
  
  public function vratPolatkyCelkovaCena()
  {
  	return $this->vratUbytovaciPoplatekCelkovaCena() + $this->vratRekreacniPoplatekCelkovaCena();
  }
  
  public function vratCenaUbytovaniSPoplatky()
  {
  	
  	return $this->vratCenaUbytovaniBezPoplatku() + $this->vratPolatkyCelkovaCena();
  }
  
   public function vratCenaStravovaniDen()
  {
     $cena = 0;
  	foreach ($this->vratStrava() as $strava)
     {
     	$cena+=  $strava->vratCelkovaCenaDen();
     }
     return $cena;
  } 

  public function vratCenaStravovaniCelkem()
  {
  	 return  $this->vratCenaStravovaniDen() * $this->vratPocetNoci();
  }
  
  public function vratCenaSluzbyCelkem()
  {
  	 $cena = 0;
  	 foreach ($this->vratSluzby() as $sluzba)
     {
     	$cena+=  $sluzba->vratCenaCelkem();
     }
     return $cena;
  }
  
  public function vratCenaCelkemVypoctena()
  {
  	  return $this->vratCenaStravovaniCelkem() + $this->vratCenaUbytovaniSPoplatky() + $this->vratCenaSluzbyCelkem();
  }
  
  public function vratCenaCelkem()
  {
  	if ($this->cenaCelkem === null)
  		$this->cenaCelkem = $this->vratCenaCelkemVypoctena();
  	return $this->cenaCelkem;  	
  }
  
  public function vratSlevaProcenta()
  {
  	if (!$this->vratCenaCelkemVypoctena()){
  		return 0;
  	}
  	return 100-100*$this->vratCenaCelkem()/$this->vratCenaCelkemVypoctena();
  }
  public function vratSlevaJednotky()
  {
  	return $this->vratCenaCelkemVypoctena() - $this->vratCenaCelkem();
  }
 
  
  public function vratUbytovaciPoplatekOsobaPobyt()
  {
    return $this->vratPocetNoci() * $this->ubytovaciPoplatekCena;
  }
  public function vratUbytovaciPoplatekCelkovaCena()
  {
  	return $this->ubytovaciPoplatekPocet * $this->vratUbytovaciPoplatekOsobaPobyt();
  }
  
  public function vratRekreacniPoplatekOsobaPobyt()
  {
  	return $this->vratPocetNoci() * $this->rekreacniPoplatekCena;
  }
  public function vratRekreacniPoplatekCelkovaCena()
  {
  	return $this->rekreacniPoplatekPocet * $this->vratRekreacniPoplatekOsobaPobyt();
  }
  
  
  public function vratPocetHostu()
  {
     $hostu = 0;
     foreach  ($this->pokojRezervace as $pokojRezervace)
    {       
       $hostu+= $pokojRezervace->vratPocetHostu();
    }   
     return $hostu;
  }
  
 /**
  * Vrací instanci Strava, která je v rezervaci a má $stravaId == $id.
  * Pokud neexistuje vrací prázdnou instanci
  * $strvaId je typ stravy
   * @param unknown $id
  * @return Strava
  */
  public function vratStravaId($id)
  {
	 foreach ($this->vratStrava() as $strava)
	 {
	 	if ($strava->vratStravaId() == $id){
	 		return $strava;
	 	}
	 } 
  	 return N::Strava();  	
  }
  
  
  public function vratPocetNoci()
  {              
    if (!$this->datumOd or !$this->datumDo) 
    return null; 
    $od = DateTime::createFromFormat('Y-m-d', $this->datumOd);
    $do = DateTime::createFromFormat('Y-m-d', $this->datumDo);    
    return FK::pocetNoci($od,$do);
  }
  
  public function vratKontakt()
  {
  	if (!$this->kontakt)
  		$this->kontakt = N::Kontakt($this->kontaktId);
  	return $this->kontakt;
  }
  
  public function vratSoucasnyStav()
  {
  	$soucasny = $this->stavy->vratPrvni();
  	if (!$soucasny)
  	{
  		$soucasny = N::StavRezervace();
  		$soucasny->nastavStavId(1);
  	}
  	return $soucasny;
  }
  
  public function vratVychoziStav()
  {
  	$vychozi = $this->stavy->vratPosledni();
  	if (!$vychozi)
  	{
  		$vychozi = N::StavRezervace();
  		$vychozi->nastavStavId(1);
  	}
  	return $vychozi;
  }
  
  /**
   * vrátí datum splatnosti
   * datum splatnosti bere z posledního stavu schváleno + 14dní
   * @return NULL|DateTime
   */
  public function vratDatumSplatnosti()
  {
  	$cas = null;
  	foreach ($this->vratStavy() as $stav)
  	{
  		if ($stav->vratStavId() == 2)
  		{
  			$cas = $stav->vratCas();
  		}
  	}
  	if ($cas)  	
  	{ 
  		$cas = DateTime::createFromFormat("Y-m-d H:i:s",$cas);
  		$cas->modify("+14 days");
  	}
  	return $cas;
  }
  
  
  
  public function ulozitDoSession()
  {      
      $_SESSION["objednavka"]["obj"] = serialize($this);      
  }
  public static function ziskatZeSession()
  {  	
  	if (isset($_SESSION["objednavka"]["obj"]))	{  		
  		$objednavka = unserialize($_SESSION["objednavka"]["obj"]);
  		return $objednavka;
  	}
  }
  
  
  public function ulozitVse()
  {
  	
  	if (!$this->pokojRezervace->vratPocet())
  	{
  	  throw new Chyba("Nelze uložit rezervaci bez vybraných pokojů");
  	}
  		if (!$this->id)	
  		{  			
  			try
  		  {  			
  			
  		  	if (!$this->kontakt)
  		  	{
  		  		throw new Chyba("Nelze uložit rezervaci bez vybraného zákazníka");
  		  	}
  		  	
  		  	DB::spustitTransakci();
  			$this->kontakt->ulozit();
  			$this->nastavKontaktId($this->kontakt->vratId());
  			parent::ulozit(); 
  			
  			foreach ($this->strava as $strava)
  			{
  				$strava->nastavRezervaceId($this->id);
  				$strava->ulozit();
  			} 
  			foreach ($this->pokojRezervace as $pokojRezervace)
  			{  				
  				$pokojRezervace->ulozit();  				
  			}
  			
  			$stavRezervace = N::StavRezervace();
  			$stavRezervace->nastavRezervaceId($this->id);
  			$stavRezervace->nastavStavId(1); 
  			$stavRezervace->ulozit();
  			new Potvrzeni("Změny byly uloženy.");
  			DB::commit();  			
  			
  		  }
  		catch(Chyba $e)
  		  {
  				$this->nastavKontaktId(null);
  				DB::rollBack();  				
  				throw new Chyba("Nastala chyba při rezervaci.");
  		  }
  		}  	  
  	  //editace
  	  else 
  	  {  	  	
  	  	try
  	  	{
  	  	DB::spustitTransakci();  	  	
  	  	parent::ulozit();
        $dotaz = "DELETE FROM pokoj_rezervace WHERE id_rezervace = :id";
        DB::dotaz($dotaz,array("id" => $this->id));   
  	  	foreach ($this->pokojRezervace as $pokojRezervace)
  	  	{
  	  		$pokojRezervace->ulozit();
  	  	}  
  	  	
  	  	$dotaz = "DELETE FROM ".Strava::vratTabulkaDB()." WHERE ".Strava::vratMapovaniDB('rezervaceId')." = :id";
  	  	//print_r($this->strava); exit();
  	  	DB::dotaz($dotaz,array("id" => $this->id));
  	  	foreach ($this->strava as $strava)
  	  	{
  	  		$strava->ulozit();
  	  	}
  	  	
  	  	DB::commit();
  	  	}
  	  	catch(Chyba $e)
  	  	{
  	  		$this->nastavKontaktId(null);
  	  		DB::rollBack();
  	  		print_r ($e);exit();
  	  		throw new Chyba("Nastala chyba při rezervaci.");
  	  	}
  	  	
  	  }
  	}
  
  
  

  public function pridatPokojRezervace($pokojId,$pocetHostu)
  {        
      $pokojRezervace = N::PokojRezervace();
      $pokojRezervace->nastavPokojId($pokojId);
      $pokojRezervace->nastavRezervace($this);     
      $pokojRezervace->nastavPocetHostu($pocetHostu);
      $this->vratPokojRezervace()->pridej($pokojRezervace,$pokojId);
  }
 
 
  public static function filtr($kolekce,$datumOd,$datumDo)
  {
     Validace::datum($datumOd, $format = 'Y-m-d');
     Validace::datum($datumDo, $format = 'Y-m-d');
    
    $dotaz = "SELECT ". self::$mapovaniDB["id"]." FROM ". self::$tabulkaDB ." WHERE " .self::$mapovaniDB["datumDo"] ." >= :datumOd AND " .self::$mapovaniDB["datumOd"] ." <= :datumDo";
    $parametry = array ("datumOd" => $datumOd, "datumDo" => $datumDo);
    self::Kolekce($kolekce,DB::dotazJedenSloupec($dotaz,$parametry),null,array());
  }
  
  
  public function zjistitExistenciFaktury()
  {
  	$soubor = __DIRFAKTURY__."faktura-".$this->id.'.pdf';
  	if ($this->id and file_exists($soubor))
  	return true;
  	return false;  	
  }
  
  public function zjistitExistenciDobropisu()
  {
  	$soubor = __DIRFAKTURY__."dobropis-".$this->id.'.pdf';
  	if ($this->id and file_exists($soubor))
  	return true;
  	return false;
  }
  
  
  /**
   * Vrací poslední Stav s hodnotou "zaplaceno" 
   * @return boolean | Stav>
   */  
  public function vratPosledniStavZaplaceni()
  {
  	if ($this->posledniStavZaplaceni === null)
  	{
  		$zaplacenaStav = false;
  		foreach ($this->vratStavy() as $stav)
	  	{	  	
	  	if ($stav->vratStavId() == 3){	  
	  		$zaplacenaStav = $stav;
	  	}
	  	$this->posledniStavZaplaceni = $zaplacenaStav;
	  }
  	}
  	return $this->posledniStavZaplaceni;
  }
  
  

  

  
  }
