<?php
class RezervaceFiltr extends Filtr
{
	
	protected $cena = null;
	   			   
	protected $rezervaceId = null;
	protected $zakaznik = null;
	protected $stavId = array();	 //array || int
	
	/** rezervace zasahují do období $datumOd do $datumdo */
	protected $datumOd = null;   //d.m.Y
	protected $datumDo = null;   //d.m.Y
	
	protected $cisloFaktury = null;
	protected $strana = null;				
	protected $pocetNaStrance = null;
	protected $priNenalezeni = null;
	protected $triditPodle = null;
	protected $triditSmer = "asc";

	
	protected $pocetPolozek = null;
	protected $kolekce = null;
	
	function __construct()
    {
   		$this->kolekce = new Kolekce('Rezervace'); 
   		$this->kolekce->nastavCallback('kolekceRezervaci', $this);
    }
   
    	
    public function nastavZakaznik($jmeno)
    {    	
		if (!$jmeno) return false;
    	Validace::delka($jmeno, 1, 255, "jméno");
    	$this->zakaznik= $jmeno;    	
    }
			
	
	public function nastavRezervaceId($id)
	{		
		if ($id and !is_numeric($id))
		{
			throw new Chyba("ID musí být číslo.");
		}
		$this->rezervaceId = $id;	
	}
	
	public function nastavTriditSmer($prom)
	{ 
		if (!$prom) return false;
		Validace::vycet($prom, array("asc","desc"), "dTřízení");
		$this->triditSmer = $prom;
	}
	
	public function nastavTriditPodle($prom)
	{
		if (!$prom) return false;
		Validace::vycet($prom, array_flip(Rezervace::vratMapovaniDB()), "Třízení");
		$this->triditPodle = $prom;
	}
	
	public function  nastavStavId($prom)
	{
		if (!$prom)	return false;
		if (is_array($prom))
		{
			foreach ($prom as $stavId)
			{
				StavRezervace::existujeStav($stavId);	
			}
		$this->stavId = $prom;				
		}
		else
		{
		  StavRezervace::existujeStav($prom);
		  $this->stavId = array($prom);
		}		
		
	}
	
		
	public function nastavPocetNaStrance($pocetNaStrance) 
	{
	   $this->pocetNaStrance = intval($pocetNaStrance);
	}
	
	public function nastavStrana($strana)
	{	
		$this->strana = intval($strana);		
	}
	
	public function nastavDatumOd($datumOd)
	{
		if (!$datumOd)	return false;
		Validace::datum($datumOd, $format = 'd.m.Y');		
		$this->datumOd = $datumOd;
	}
	
	public function nastavDatumDo($datumDo)
	{
		if (!$datumDo)	return false;
		Validace::datum($datumDo, $format = 'd.m.Y');		
		$this->datumDo = $datumDo;
	}




 public function kolekceRezervaci()
 {	
	
	$podminka = array();
	$skupina = null;
	$pripoj = array();
	$triditPodle = null;	
	$parametry = array();
	$dotazPodminka = null;
	
	
	
	
	if ($this->rezervaceId)
	{		
		$podminka[] = "id_rezervace = :rezervaceId";
		$parametry["rezervaceId"] = $this->rezervaceId;
	}

	if ($this->zakaznik)
	{		
		$pripoj['kontakt'] ="id_kontakt";
	    $podminka[]= "(jmeno LIKE :jmeno OR prijmeni LIKE :jmeno OR organizace LIKE :jmeno)";
		$parametry["jmeno"] = "%".$this->zakaznik."%";
	}	
	
		
	if ($this->datumOd)
	{
		$podminka[]= "rezervace.datum_do >= :datumOd";
		$datum= DateTime::createFromFormat('d.m.Y', $this->datumOd);	
		$parametry["datumOd"] = $datum->format('Y-m-d');	
	}
	
	if ($this->datumDo)
	{
		$podminka[]= "rezervace.datum_od  <= :datumDo";
		$datum= DateTime::createFromFormat('d.m.Y', $this->datumDo);	
		$parametry["datumDo"] = $datum->format('Y-m-d');
	}	
	
	if ($this->stavId)
	{		
		$podminka[]= "(SELECT id_stav FROM stav_rezervace WHERE rezervace.id_rezervace = stav_rezervace.id_rezervace ORDER BY cas DESC LIMIT 1) IN(".implode(",",array_values($this->stavId)).")";
	}
	if ($this->cisloFaktury)
	{
		$podminka[] = "cislo_faktury = :cisloFaktury";
		$parametry["cisloFaktury"] = $this->cisloFaktury;
	}
	if ($this->triditPodle)
	{		
		$mapovani = Rezervace::vratMapovaniDB();
		$triditPodle = "ORDER BY ". $mapovani[$this->triditPodle]." ".$this->triditSmer;
	}
	
		
	
	$join = null;
	foreach ($pripoj as $tabulka => $sloupec)
	{
	  $join.= "JOIN $tabulka USING ($sloupec)";
	}
	foreach ($podminka as  $klic => $p)
	{
		if ($klic != 0) $dotazPodminka.= " AND ";
		$dotazPodminka.= $p;
	}
	$dotazPodminka = ($dotazPodminka) ? "WHERE ". $dotazPodminka  : null; 
	$skupina = ($skupina) ? "GROUP BY id_rezervace HAVING ".$skupina : null;

	$dotaz = "SELECT distinct id_rezervace FROM rezervace $join $dotazPodminka $skupina $triditPodle";	
	
	//print_r ($parametry);
	
	$poleID = DB::dotazJedenSloupec($dotaz,$parametry);
	$this->pocetPolozek = count($poleID);

	//print_r($poleID);
    //echo $dotaz; 
	
 		 
	if ($this->strana and $this->pocetNaStrance)	
	{
		
		$zaznam = ($this->strana-1) * $this->pocetNaStrance;
		$poleID = array_slice($poleID,$zaznam,$this->pocetNaStrance);
	}
	elseif ($this->pocetNaStrance)
	{
		$poleID = array_slice($poleID,0,$this->pocetNaStrance);
		
	} 	
		
	
  if (count($poleID) == 0)
	{
		
		if ($this->priNenalezeni == "404")
		{
		 FK::presmerovat(__WEB__."404");
		}
		elseif ($this->priNenalezeni == "return")
		{
			return false;
		}   		 
		new Potvrzeni("Nebyly nalezeny žádné odpovídající rezervace.");
		
	}
	else 
	{
		Rezervace::kolekce($this->kolekce,$poleID,null);	
	}
	
	
	
 }
} 
 
	