<?php
class  Zakaznik extends BO2
{ 
 private $heslo = null; //heslo lze používat jen v rámci třídy

//načítáno dle potřeby
 private $obnovaHeslaToken = false; 
 private $obnovaHeslaCas = false;
  	
 protected $id = null;
 protected $email = null;
 protected $kontaktId = null;
  
 protected $kontakt = null;
 protected static $kolekce = null;

 protected static $tabulkaDB = "zakaznik"; 
 
 //heslo není načítáno z DB
 protected static $mapovaniDB = array ( 
   "id" => "id_zakaznik",
   "kontaktId" => "id_kontakt",
   "email" => "email",
 
   );
   
 function __construct($id=null)
 {
 	parent::__construct($id);
 	
 	$this->rezervace = new Kolekce('Rezervace');
 	$this->rezervace ->nastavCallback('kolekceSouvisejicich', $this);
 }
 
 
 /**
  * zajištuje že nebudou dva zákazníci se stejnými e-maily
  * @param string $email
  * @throws Chyba
  */ 
 public function nastavEmail($email)
 {
      Validace::email($email);
 	  $dotazId =null;
 	
	  if ($this->id)
	  {
	  	$dotazId = " and ".self::$mapovaniDB['id']." <> :id ";
	  	$parametry["id"] = $this->id;
	  } 	 
	  	
	  $parametry["email"] = $email;     
	  $dotaz = "SELECT * FROM zakaznik WHERE ".self::$mapovaniDB['email']."  = :email $dotazId LIMIT 1";	
	  
	  if(DB::dotazPocet($dotaz,$parametry) != 0)
	  {
	   throw new Chyba('Registrace se stejným e-mailem již existuje.');
	  }
	  $this->email = $email;
 }
 
 
 
 public function nastavHeslo($heslo,$heslo2)
 {
 	  Validace::delka($heslo, 6, 255, "Heslo"); 	
	  if ($heslo != $heslo2)
	  {
	   throw new Chyba('Hesla se neshodují.');
	  }
	  else
	  {	    
	  	$this->heslo = password_hash($heslo, PASSWORD_DEFAULT);
	  }
 }
 
 public function vratKontakt()
 { 	
 	if (!$this->kontakt)   
 	$this->kontakt = N::Kontakt($this->kontaktId);
 	return $this->kontakt;
 } 
 

 /**
  * přetěžuje rodičovskou metodu, heslo ukladá jen při vytvoření uživatele nebo změně
  * dotazy se vytvářejí zde také kvůli absenci hesla při mapování do db
  * ukládá také Kontakt  * 
  */ 
 
 public function ulozit()
 { 
 	 try 
 	  { 
 	  	$this->vratKontakt();
 		DB::spustitTransakci(); 		
 		$this->kontakt->ulozit(); 
 		$this->nastavKontaktId($this->kontakt->vratId());
 		$parametry = array("email" => $this->email);
 		
 		//nový zákazník
 		if (!$this->id)
 		{
 		   if (!$this->heslo)
  			{
  				throw new Chyba("Nebylo nastaveno heslo.");
  			}
 			$parametry["heslo"] = $this->heslo;
 			$parametry["kontaktId"] = $this->kontaktId;
 			$dotaz = "INSERT INTO zakaznik (email,heslo,id_kontakt) VALUES(:email,:heslo,:kontaktId)";
 		}
 		//stávající zákazník
 		else
 		{
 			$parametry["id"] = $this->id;
 			//změna hesla
 			if ($this->heslo)
 			{
 				$parametry["heslo"] = $this->heslo;
 				$dotaz = "UPDATE zakaznik SET email=:email, heslo=:heslo WHERE id_zakaznik=:id";
 			}
 			else 
 			{
 				$dotaz = "UPDATE zakaznik SET email=:email  WHERE id_zakaznik=:id";
 			} 				
 		}	
 		
 		DB::dotaz($dotaz,$parametry);
 		DB::commit(); 	
 	  }
 	  catch(Chyba $e)
 	  { 	  	
 	  	DB::rollBack();
 	  	throw new Chyba("Nastala chyba při ukládání zákazníka.");
 	  }
 }
 
 /**
  * Zrušit vazby na rezervace, smazat zákazníka a kontakt 
  * @throws Chyba
  */ 
 public function smazat()
 {
 	try
 	{
 		DB::spustitTransakci();	
 		foreach ($this->vratRezervace() as $rezervace)
 		{
 		  $rezervace->nastavZakaznikId(null);
 		  $rezervace->ulozit();
 		}	
 		
 		parent::smazat();
 		$kontakt = $this->vratKontakt();
 		$kontakt->smazat(); 		
 		DB::commit();
 	}
 	catch(Chyba $e)
 	{
 		DB::rollBack();
 		throw new Chyba("Nastala chyba při mazání zákazníka.");
 	}
 }
 
 /**
  * Vrací obj. Zákazník podle zadaného e-mailu
  * @param unknown $email
  * @throws Chyba
  * @return Zakaznik
  */
 public static function vratZakaznikaDleEmailu($email)
 {
 	Validace::email($email);
	$parametry["email"] = $email;
 	$dotaz = "SELECT id_zakaznik FROM zakaznik WHERE email=:email  LIMIT 1";
	$radek = DB::dotazJeden($dotaz,$parametry);
	 if (!$radek){
	 	Throw new Chyba("Zákazník s tímto e-mailem neexistuje.");
	 }
	 return N::Zakaznik($radek["id_zakaznik"]); 
 }
 
 /**
  * Vytváří a ukládá do db token pro obnovu hesla
  * Při nastaveném parametru zrušit odebírá token z db
  * @return string - nešifrovaný token
  * @param string $zrusit
  */
 public function pozadavekNaObnovuHesla($zrusit = null)
 {  		
 	$tokenNesifrovany = null;
 	if (!$zrusit)
 	{ 		
 		$tokenNesifrovany = FK::nahodnyToken($delka = 30);
 		$this->obnovaHeslaToken = password_hash($tokenNesifrovany, PASSWORD_DEFAULT); 
 		$casVytvoreni = new Datetime();
 		$this->obnovaHeslaCas = $casVytvoreni->format("Y-m-d H:i:s"); 		
 	}
 	else 
 	{
 		$this->obnovaHeslaToken = null;
 		$this->obnovaHeslaCas = null; 	
 	} 	
  	$parametry = array("id" => $this->id, "obnovaHeslaCas" => $this->obnovaHeslaCas, "obnovaHeslaToken" => $this->obnovaHeslaToken );
 	$dotaz = "UPDATE zakaznik SET oh_cas=:obnovaHeslaCas,oh_token=:obnovaHeslaToken WHERE id_zakaznik=:id";
 	DB::dotaz($dotaz,$parametry); 
 	return 	$tokenNesifrovany;
 }
 
 
 /**
  * Vrací token pro obnovu hesla v zašifrované podobě
  */  
 public function vratObnovaHeslaToken()
 {
 	if ($this->obnovaHeslaToken === false)
 	{
 		$parametry["id"] = $this->id;
 		$dotaz = "SELECT oh_token , oh_cas FROM zakaznik WHERE id_zakaznik = :id LIMIT 1";
 		$radek = DB::dotazJeden($dotaz,$parametry); 		
 		if ($radek and $radek["oh_token"])
 		{
 			$this->obnovaHeslaToken =  $radek["oh_token"];
 			$this->obnovaHeslaCas = $radek["oh_cas"];
 		}
 		else 
 		{
 			$this->obnovaHeslaToken = false;
 		} 		
 	} 
 	return $this->obnovaHeslaToken;	
 }
 public function vratObnovaHeslaCas()
 {
 	$this->vratObnovaHeslaToken();
 	return $this->obnovaHeslaCas;
 }
 
 
 
}
